import os
import json
import subprocess

from .database import get_database, get_connection_info
from .helpers import is_systemd

CAPTURECAST_SERVICE = "cattura-capturecast"
LEGACY_CAPTURECAST_SERVICE = "catturaCB"

class CaptureCast (object):
    def __init__ (self, context):
        self.context = context
        self.path = context.get_path(context.config.cattura_path)
    
    def get_settings (self):
        """Return CaptureCast settings, if there are any. Return None if there are none."""
        try:
            with open(self.get_settings_file(), "r") as confFile:
                return json.loads(confFile.read())
        except IOError: pass
        except ValueError: pass
    
    def is_running (self):
        """Return true if CaptureCast running, false otherwise."""
        try:
            self.context.call(["service", self.get_service(), "status"])
            return True
        except subprocess.CalledProcessError:
            return False
    
    def start (self):
        """Start CaptureCast."""
        return self.context.call_privileged(["service", self.get_service(), "start"])
    
    def stop (self):
        """Stop CaptureCast."""
        return self.context.call_privileged(["service", self.get_service(), "stop"])
    
    def restart (self):
        """Restart CaptureCast, if it is running."""
        if is_systemd():
            return self.context.call_privileged(["systemctl", "reload-or-try-restart", "{}.service".format(self.get_service())])
        else:
            return self.context.call_privileged(["service", self.get_service(), "try-restart"])
    
    def save_settings (self, settings):
        """Save CaptureCast settings. If configuration directory does not exist, create it."""
        self.context.call_privileged(["mkdir", "-p", self.get_conf_directory()])
        return self.context.write_privileged(json.dumps(settings), self.get_settings_file())
    
    def get_conf_directory (self):
        """Get path to CaptureCast configuration directory."""
        return os.path.join(self.path, "conf")
    
    def get_plugins_directory (self):
        """Get path to CaptureCast plugins directory."""
        return os.path.join(self.path, "plugins")
    
    def get_storage_directory (self):
        """Get path to CaptureCast storage directory."""
        return os.path.join(self.path, "storage")
    
    def get_settings_file (self):
        """Get path to CaptureCast settings file."""
        return os.path.join(self.get_conf_directory(), "private_settings.json")
    
    def get_version (self):
        """Discover the version of CaptureCast currently installed. Returns None if not installed."""
        version_file = os.path.join(self.path, "VERSION")
        
        if os.path.exists(version_file):
            with open(version_file) as version_file:
                version_data = version_file.read().strip()
                try:
                    return json.loads(version_data)["version"]
                except ValueError:
                    return version_data

    def get_database (self):
        """Return database object for this instance of CaptureCast."""
        return get_database(self.context, get_connection_info(self.get_settings()))

    def get_service (self):
        if self.get_version().startswith("4."):
            return LEGACY_CAPTURECAST_SERVICE
        
        return CAPTURECAST_SERVICE