from datetime import datetime
import argparse
from .helpers import random_string

options = {
    # Available user actions.
    "actions": {
        "setup-os": {
            "title": "Setup OS",
            "description": "Set up the OS: create user accounts and perform OS configuration.",
            "abbreviation": "so"
        },
        "register": {
            "title": "Register CaptureCast Recorder",
            "abbreviation": "rs"
        },
        "install": {
            "title": "Install CaptureCast Platform",
            "description": "Implies --register and --install-drivers; may also imply --setup-os and --install-extras",
            "abbreviation": "i"
        },
        "install-drivers": {
            "title": "Install Drivers Only",
            "abbreviation": "id"
        },
        "install-extras": {
            "title": "Install Extras",
            "description": "Install extra packages (such as web browser and video player)",
            "abbreviation": "ie"
        },
        "check": {
            "title": "Check Installation",
            "description": "Run checks to make sure things installed correctly: drivers, plugins, video devices, etc.",
            "abbreviation": "ch"
        },
        "check-updates": {
            "title": "Check for Updates",
            "description": "Check for available updates to Cattura software.",
            "abbreviation": "cu"
        },
        "uninstall": {
            "title": "Uninstall CaptureCast Platform",
            "abbreviation": "ui"
        },
        "downgrade": {
            "title": "Downgrade CaptureCast Platform",
            "description": "Downgrade CaptureCast Platform to older version. Technically this performs an uninstall and then install. See also --reinstall",
            "abbreviaton": "dg"
        },
        "reinstall": {
            "title": "Reinstall CaptureCast Platform",
            "description": "Reinstall CaptureCast Platform. This performs an uninstall and then reinstall. See also --downgrade",
            "abbreviation": "re"
        },
        "unregister": {
            "title": "Unregister CaptureCast Recorder",
            "description": "Also removes repo configuration. Does not imply --uninstall or --wipe-data.",
            "abbreviation": "ur"
        },
        "wipe-data": {
            "title": "Wipe stored data",
            "description": "Wipes storage directory and database. Does not imply --uninstall or --unregister.",
            "abbreviation": "wd"
        },
        "nuke": {
            "title": "Nuke CaptureCast",
            "description": "Wipe all traces of CaptureCast from the recorder; implies --uninstall, --wipe-data, --unregister",
            "abbreviation": "nu"
        },
        "install-command-center": {
            "title": "Install Cattura Command Center",
            "description": "Installs the Cattura Command Center.",
            "abbreviation": "icc"
        },
        "get-registration-info": {
            "title": "Get registration info",
            "description": "Retrieve registration info (such as license key and expiration)",
            "abbreviation": "gr"
        },
        "check-key": {
            "title": "Check license key",
            "description": "Check the validity of the passed-in license key.",
            "abbreviation": "ck"
        },
        "create-user": {
            "title": "Create user",
            "description": "Create and configure the cattura-customer user.",
            "abbreviation": "cru"
        },
        "refresh-user": {
            "title": "Refresh user",
            "description": "Remove and re-create customer user with latest configuration",
            "abbreviation": "reu"
        },
        "remove-user": {
            "title": "Remove customer user",
            "description": "Remove the cattura-customer user.",
            "abbreviation": "ru"
        },
        "remount": {
            "title": "Remount root partition",
            "description": "Remount the root partition as writable. Does nothing if the root is not read-only.",
            "abbreviation": "rm"
        },
        "perform-updates": {
            "title": "Perform Cattura Updates",
            "description": "Updates all packages from Cattura repositories. This is also done by --install if CaptureCast is already installed.",
            "abbreviation": "pu"
        }
    },
    
    # Repository sources.
    # These are shorthands for known repos.
    "repos": {
        "prod": "production",
        "legacy": "https://repositories.catturavideo.com/production/current",
        "v4": "https://repositories.catturavideo.com/production/current",
        "4.12.10": "https://repositories.catturavideo.com/4.12.10"
    },
    
    # Device drivers such as magewell and datapath
    "drivers": {
        "magewell": {"title": "Magewell Pro Capture Drivers", "packages": "cattura-magewell-pro-capture-drivers", "devices": [
            "Device 1cd7:0003", "Device 1cd7:0006", "Device 1cd7:0008", "Device 1cd7:0009",
            "Device 1cd7:0010", 
            "Nanjing Magewell Electronics Co., Ltd. Device 0004",
            "Nanjing Magewell Electronics Co., Ltd. Device 0008"
        ]},
        "magewell-eco": {"title": "Magewell Eco Capture Drivers", "packages": "cattura-magewell-eco-capture-drivers"}
    },

    # Publishing destinations such as kaltura, youtube, etc
    "publishers": {
        "standard": {"title": "Standard Publishers", "packages": "cattura-standard-publishing-plugins"},
        "kaltura": {"title": "Kaltura", "packages": "cattura-kaltura-plugin"},
        "youtube": {"title": "YouTube", "packages": "cattura-youtube-plugin"},
        "google-drive": {"title": "Google Drive", "packages": "cattura-google-drive-plugin"},
        "opencast": {"title": "Opencast", "packages": "cattura-opencast-publishing-plugin"},
        "panopto": {"title": "Panopto", "packages": "cattura-panopto-plugin"},
        "vimeo": {"title": "Vimeo", "packages": "cattura-vimeo-plugin"},
        "sftp": {"title": "SFTP", "packages": "cattura-sftp-plugin"},
        "ftp": {"title": "FTP", "packages": "cattura-ftp-plugin"},
        "box": {"title": "Box", "packages": "cattura-box-plugin"}
    },

    # User-facing features such as the editor, scheduler, device capture, etc
    "features": {
        # Standard installation profiles. This is what most people will want.
        "standard": {"title": "Standard Features", "packages": "cattura-standard-features"},
        "editing-station": {"title": "Editing Station Profile", "packages": "cattura-editing-station-features"},

        # Standard collections of plugins. This is granular enough for most people.
        "standard-web": {"title": "Standard Web Plugins", "packages": "cattura-standard-web-plugins"},
        "standard-capture": {"title": "Standard Capture Plugins", "packages": "cattura-standard-capture-plugins"},
        "standard-encoding": {"title": "Standard Encoding Plugins", "packages": "cattura-standard-encoding-plugins"},
        "standard-analysis": {"title": "Standard Analysis Plugins", "packages": "cattura-standard-analysis-plugins"},
        "standard-metadata": {"title": "Standard Metadata Plugins", "packages": "cattura-standard-metadata-plugins"},
        "standard-modification": {"title": "Standard Modification Plugins", "packages": "cattura-standard-modification-plugins"},
        "standard-notifiers": {"title": "Standard Notifier Plugins", "packages": "cattura-standard-notifier-plugins"},
        "standard-auth": {"title": "Standard Authentication Plugins", "packages": "cattura-standard-auth-plugins"},
        "standard-scheduling": {"title": "Standard Scheduling Plugins", "packages": "cattura-standard-scheduling-plugins"},

        # A-la-carte customization of specific features. This is advanced stuff.
        "editor": {"title": "Editor", "packages": "cattura-media-package-editor-plugin"},
        "scheduler": {"title": "Scheduler", "packages": "cattura-scheduler-plugin"},
        "webhook": {"title": "Webhooks", "packages": "cattura-webhook-plugin"},
        "email": {"title": "Email Notifications", "packages": "cattura-email-notification-plugin"},
        "compositor": {"title": "Frame Compositor", "packages": "cattura-frame-compositor-plugin"},
        "mediainfo": {"title": "MediaInfo", "packages": "cattura-mediainfo-plugin"},
        "live-preview": {"title": "Live Preview", "packages": "cattura-live-preview-plugin"},
        "learning-engine": {"title": "Learning Engine Integration", "packages": "cattura-learning-engine-plugin"},
        "thumbnail-refresher": {"title": "Device Thumbnail Refresher", "packages": "cattura-device-thumbnail-refresh-plugin"},
        "web-interface": {"title": "Web User Interface", "packages": "cattura-web-ui-plugin"},
        "qt-faststart": {"title": "QT Faststart", "packages": "cattura-qt-faststart-plugin"},
        "v4l": {"title": "Video4Linux Capture", "packages": "cattura-v4l-capture-plugin"},
        "alsa": {"title": "ALSA Capture", "packages": "cattura-alsa-capture-plugin"},
        "writer": {"title": "Writer", "packages": "cattura-xuggler-writer-plugin"},
        "rtmp": {"title": "RTMP Streaming", "packages": "cattura-rtmp-streaming-plugin"},
        "importer": {"title": "Media Package Importer", "packages": "cattura-media-package-importer-plugin"},
        "ppm": {"title": "PPM Viewer", "packages": "cattura-audio-level-viewer-plugin"},
        "log-downloader": {"title": "Log Downloader", "packages": "cattura-log-downloader-plugin"},
        "graph-visualizer": {"title": "Graph Visualizer", "packages": "cattura-graph-visualizer-plugin"},
        "ocr": {"title": "OCR", "packages": "cattura-tesseract-ocr-plugin"},
        "scene-change-detector": {"title": "Scene Change Detector", "packages": "cattura-shotdetect-plugin"},
        "audio-dumper": {"title": "Audio Dumper", "packages": "cattura-audio-dumper-plugin"},
        "file-upload-service": {"title": "File Upload Service", "packages": "cattura-web-upload-service-plugin"},
        "web-api": {"title": "Web API", "packages": ["cattura-web-api-plugin", "cattura-graph-builder-api-pugin"]},

        # Integration with other third-party services
        "kaltura-scheduler": {"title": "Kaltura Scheduler", "packages": "cattura-kaltura-scheduler-plugin"},
        "rev-transcription": {"title": "Rev Transcription", "packages": "cattura-rev-plugin"}
    },

    # Backend services. These should generally be left at default.
    "backend": {
        "nginx": {"title": "Nginx Frontend Server", "packages": "cattura-nginx"}
    },
}

# Defaults for feature options. Used by the install_features task.
defaults = {
    "repo": "production",
    "features": ["standard"],
    "publishers": ["standard"],
    "drivers": ["magewell", "magewell-eco"],
    "backend": [key for key in options["backend"].keys()]
}

def argparser ():
    parser = argparse.ArgumentParser(description="Cattura CaptureCast Installer, version {}".format("2.0.2"))

    # Actions
    for action_name, value in options["actions"].items():
        help = value["title"]
        if "description" in value:
            help = "{}: {}".format(help, value["description"])
        names = ["--{}".format(action_name)]
        if "abbreviation" in value:
            names = ["-{}".format(value["abbreviation"])] + names
        parser.add_argument(*names, type=str, nargs="*", help=help)

    # Action options
    parser.add_argument("-l", "--loglevel", type=str, help="Log level (default INFO)", nargs="?")
    parser.add_argument("-lf", "--logfile", type=str, help="Log file (default installer-<datetime>.log)", nargs="?")
    parser.add_argument("-d", "--dry", action="store_true", help="Dry run: don't make any changes to the system")
    parser.add_argument("-j", "--json", action="store_true", help="Output result value as json")
    parser.add_argument("-m", "--mode", type=str, help="Mode to run installer in (default terminal)", nargs="?")
    parser.add_argument("-y", "--yes", action="store_true", help='Automatically answer "yes" to any prompts')
    parser.add_argument("-t", "--target", type=str, help="Target filesystem for installation", nargs="?")
    parser.add_argument("-ra", "--remount-as", type=str, help="Mode to remount root filesystem to (default rw)", nargs="?")

    # Registration (Lemonade Stand) settings
    parser.add_argument("--registration-url", type=str, help="URL for registration", nargs="?")
    parser.add_argument("--key", type=str, help="license key for registration", nargs="?")

    # Repository to track
    parser.add_argument("--repo", "--version", type=str, help="repository name or URL", nargs="?")

    # Database settings
    parser.add_argument("--database-type", type=str, help="type of database", nargs="?")
    parser.add_argument("--database-host", type=str, help="database host", nargs="?")
    parser.add_argument("--database-user", type=str, help="database username", nargs="?")
    parser.add_argument("--database-name", type=str, help="database name", nargs="?")
    parser.add_argument("--database-password", type=str, help="database password", nargs="?")

    # Feature options
    parser.add_argument("--publishers", type=str, nargs="*", help="Publishers to install (default is standard publishers)")
    parser.add_argument("--features", type=str, nargs="*", help="Features to install (default is standard features)")
    parser.add_argument("--backend", type=str, nargs="*", help="Additional backend services to install (default is standard backend services)")
    parser.add_argument("--drivers", type=str, nargs="*", help="Drivers to install (default is all drivers)")

    # Installation options
    parser.add_argument("-sa", "--standalone", action="store_true", help="Only install CaptureCast by itself. Implies --no-os-extras and empty --backend and --drivers selections")
    parser.add_argument("-noe", "--no-os-extras", action="store_true", help="Do not install additional OS configs or tools")
    parser.add_argument("-rf", "--restore-from", type=str, help="Backup file to restore from after install", nargs="?")
    
    # Update options
    parser.add_argument("-nr", "--norestart", action="store_true", help="Do not prompt to restart after update")
    parser.add_argument("-nc", "--nochangelogs", action="store_true", help="Do not fetch changelogs")
    parser.add_argument("--update-from", type=str, nargs="*", help="Repositories to update from (default is all repos)")

    return parser

def parse_args (args):
    return from_parsed_args(argparser().parse_args(args))

class Config (object):
    def __init__ (self):
        self.database = {
            "host": "127.0.0.1",
            "type": "postgresql",
            "user": "cattura",
            "password": random_string(32),
            "name": "cattura"
        }

        # Lemonade Stand settings
        self.registration_url = "https://registration.catturavideo.com/" # Lemonade Stand registration URL
        self.key = None # key will be prompted for, if necessary

        # Cattura repo settings
        # If this is not None, then the repo will always be changed (e.g. from
        # production to beta)
        self.repo = None

        # Whether or not to install extra OS tools such as teamviewer, chrome, etc
        # These are considered part of the capturecast distribution to most users
        # but if capturecast is being installed in a minimal or server environment
        # then it may not be desirable.
        self.install_os_extras = True

        # Features to install
        # If these are None (default) then the install_features task will install
        # default features as specified in the `defaults` field of this module.
        # This is so we can tell when the user explicitly requests to install
        # a feature, since we will always install features requested by the user
        # (even if the platform is already installed)
        self.cattura_path = "/var/cattura"
        self.drivers = None
        self.publishers = None
        self.features = None
        self.backend = None
        
        # Action settings
        self.dry = False
        self.actions = [{"action": "install", "args": []}]
        self.loglevel = "info"
        self.logfile = "installer-{}.log".format(str(datetime.now()).replace(" ", "-"))
        self.json = False
        self.mode = "terminal"
        self.yes = False
        self.target = None
        self.remount_as = "rw"
        
        # Backup argument (for install)
        self.backup = None
        
        # Update settings
        self.norestart = False
        self.nochangelogs = False
        self.update_from = None

    def __str__ (self):
        return repr(vars(self))

def from_parsed_args (args):
    args = dict([item for item in vars(args).items() if item[1] is not None])
    config = Config()

    # Lemonade Stand
    config.registration_url = args.get("registration_url", config.registration_url)
    config.key = args.get("key", config.key)

    # Repo
    config.repo = args.get("repo")

    # OS extras setting
    # The default value of this setting is true, and the argument flag is actually
    # the inverse (install only capturecast).
    if "no_os_extras" in args:
        config.install_os_extras = not args["no_os_extras"]

    # Standalone flag
    # This disables several things that would normally be installed alongside
    # CaptureCast, including other backend services (such as nginx), OS extras
    # and config, and drivers.
    standalone = args.get("standalone", False)
    if standalone:
        config.install_os_extras = False
        config.drivers = []
        config.backend = []

    # Database settings
    config.database["type"] = args.get("database_type", config.database["type"])
    config.database["host"] = args.get("database_host", config.database["host"])
    config.database["user"] = args.get("database_user", config.database["user"])
    config.database["name"] = args.get("database_name", config.database["name"])
    config.database["password"] = args.get("database_password", config.database["password"])

    # Features to install
    config.features = args.get("features", config.features)
    config.publishers = args.get("publishers", config.publishers)
    config.backend = args.get("backend", config.backend)
    config.drivers = args.get("drivers", config.drivers)
    
    # Action settings
    config.dry = args.get("dry", config.dry)
    config.loglevel = args.get("loglevel", config.loglevel)
    config.logfile = args.get("logfile", config.logfile)
    config.json = args.get("json", config.json)
    config.mode = args.get("mode", config.mode)
    config.yes = args.get("yes", config.yes)
    config.backup = args.get("restore_from", config.backup)
    config.norestart = args.get("norestart", config.norestart)
    config.target = args.get("target", config.target)
    config.nochangelogs = args.get("nochangelogs", config.nochangelogs)
    config.update_from = args.get("update_from", config.update_from)
    config.remount_as = args.get("remount_as", config.remount_as)

    # Parse actions
    default_actions = config.actions
    config.actions = []
    for action_name, value in options["actions"].items():
        action_argument_name = action_name.replace("-", "_")
        if not args.get(action_name.replace("-", "_"), None) is None:
            config.actions.append({
                "action": action_name,
                "args": args.get(action_argument_name)
            })
    config.actions = config.actions if config.actions else default_actions

    return config

default = Config()
