import os
import subprocess
from time import sleep

from .helpers import is_root
from .commands import read_file, write_file, install, remove, debconf, chroot, add_apt_repository
from .config import Config
from .pipe import Pipes

def log_call (name, args, kwargs):
    if kwargs:
        return "{}({}, {})".format(name, repr(args)[1:-1], repr(kwargs)[1:-1])
    else:
        return "{}({})".format(name, repr(args)[1:-1])

def logger_callback (logger):
    def logger_callback (line):
        logger.info(line.replace("\r", ""))
    return logger_callback

def print_callback (line):
    print(line)

def collect_lines_callback ():
    lines = []
    def callback (line):
        lines.append(line)
    return (lines, callback)

def prepend_to_command (prefix, args, kwargs):
    args = list(args)
    if kwargs.get("shell", False):
        args[0] = ["/bin/sh", "-c", args[0]]
        del kwargs["shell"]
    
    try:
        args[0] = [prefix] + args[0]
    except TypeError:
        args[0] = "{} {}".format(prefix, args[0])
    return (args, kwargs)

def create_apt_env ():
    apt_env = os.environ.copy()
    apt_env["DEBIAN_FRONTEND"] = "noninteractive"
    return apt_env

class Context (object):
    def __init__ (self, config=None, logger=None):
        self.config = config if config else Config()
        self.logger = logger
    
    def get_hostname (self):
        return self.check_output_chroot("hostname").decode().strip()

    def get_path (self, *parts):
        if self.config.target:
            return os.path.join(self.config.target, *parts)
        return os.path.join(*parts)
        
    def install (self, *args, **kwargs):
        if self.logger:
            self.logger.info(log_call("install", args, kwargs))
        return self.try_apt_command(install(args, **kwargs))
    
    def debconf (self, *args, **kwargs):
        if self.logger:
            self.logger.info(log_call("debconf", args, kwargs))
        return self.try_apt_command(debconf(*args, **kwargs), shell=True)
    
    def add_apt_repository (self, *args, **kwargs):
        if self.logger:
            self.logger.info(log_call("add_apt_repository", args, kwargs))
        return self.try_apt_command(add_apt_repository(args, **kwargs))
    
    def remove (self, *args, **kwargs):
        if self.logger:
            self.logger.info(log_call("remove", args, kwargs))
        return self.try_apt_command(remove(args, **kwargs))
    
    def apt_update (self):
        return self.try_apt_command(["apt-get", "-y", "update", "--force-yes"])

    def try_apt_command (self, command, env=None, shell=False):
        if not env:
            env = create_apt_env()

        attempt = 0
        while True:
            (err, err_callback) = collect_lines_callback()
            
            try:
                return self.call_chroot(command, err_callback=err_callback, stdin=subprocess.PIPE, env=env, shell=shell)
            except subprocess.CalledProcessError as e:
                err = "\n".join(err)
                if "Resource temporarily unavailable" in err:
                    attempt = attempt + 1
                    self.logger.warning(
                        "Failed to run apt/dpkg command (error code %s, attempt %s), probably because another apt/dpkg process is running. Will try again...",
                        e.returncode,
                        attempt
                    )
                    sleep(5)
                else:
                    raise e
    
    def read_privileged (self, *args, **kwargs):
        command = read_file(*args, **kwargs)
        if not is_root():
            command = [self.get_sudo()] + command
        
        return subprocess.check_output(command).decode()
    
    def write_privileged (self, *args, **kwargs):
        return self.call_privileged(write_file(*args, **kwargs))
    
    def call (self, *args, **kwargs):
        if self.logger:
            self.logger.info(log_call("call", args, kwargs))
        
        if self.config.dry:
            return 0
        
        callback = logger_callback(self.logger) if self.logger else print_callback
        
        out_callbacks = [callback]
        err_callbacks = [callback]
        
        if "out_callback" in kwargs:
            out_callbacks.append(kwargs["out_callback"])
            del kwargs["out_callback"]
        
        if "err_callback" in kwargs:
            err_callbacks.append(kwargs["err_callback"])
            del kwargs["err_callback"]
        
        kwargs["stdout"] = subprocess.PIPE
        kwargs["stderr"] = subprocess.PIPE
        process = subprocess.Popen(*args, **kwargs)
        retval = Pipes(process, out_callbacks=out_callbacks, err_callbacks=err_callbacks).wait()
        
        if retval != 0:
            raise subprocess.CalledProcessError(retval, args[0])
        
        return retval

    def call_privileged (self, *args, **kwargs):
        if not is_root():
            (args, kwargs) = prepend_to_command(self.get_sudo(), args, kwargs)
        
        return self.call(*args, **kwargs)
    
    def call_chroot (self, *args, **kwargs):
        return self.call_privileged(chroot(self.config.target, *args), **kwargs)

    def check_output_chroot (self, *args, **kwargs):
        if not self.config.target:
            return subprocess.check_output(*args, **kwargs)
        
        args[0] = chroot(self.config.target, args[0])
        if not is_root():
            (args, kwargs) = prepend_to_command(self.get_sudo(), args, kwargs)
        
        return subprocess.check_output(*args, **kwargs)

    def input (self, prompt):
        if self.is_graphical():
            return subprocess.check_output(["zenity", "--entry", "--text={}".format(prompt)]).decode().strip()
        else:
            return input(prompt)
    
    def ask_yesno (self, prompt):
        if self.config.yes:
            self.logger.info("{}: Y".format(prompt))
            return True
        elif self.is_graphical():
            return subprocess.call(["zenity", "--question", "--text={}".format(prompt)]) == 0
        else:
            return input(prompt).lower().startswith("y")
    
    def is_graphical (self):
        return "graphic" in self.config.mode
    
    def get_sudo (self):
        return "pkexec" if self.is_graphical() else "sudo"
