import os
from io import StringIO
from urllib.parse import urlparse
import urllib.request
#from urllib.error import URLError
import gzip
import base64
from functools import total_ordering

DPKG_STATUS = os.path.join("/", "var", "lib", "dpkg", "status")
PACKAGES = "Packages"
PACKAGES_GZ = "Packages.gz"
RELEASE = "Release"

class Repo (object):
    def __init__ (self, type, options, url, dist, component=None):
        self.type      = type
        self.url       = url
        self.dist      = dist
        self.options   = options
        self.component = component

    def packages (self):
        packages_root = self.url

        if self.dist and (not self.dist == "/"):
            packages_root = "{}/dists/{}".format(packages_root, self.dist)

        release_data = urlopen("{}/{}".format(packages_root, RELEASE)).read().decode()

        for line in release_data.split("\n"):
            packages_path     = line.strip().split(" ")[-1]
            packages_filename = os.path.basename(packages_path)
            if packages_filename in [PACKAGES, PACKAGES_GZ]:
                packageData = urlopen("{}/{}".format(packages_root, packages_path)).read()

                if packages_filename == PACKAGES_GZ:
                    packageData = gzip.GzipFile(fileobj=StringIO(packageData)).read()

                return parsePackageList(packageData.decode())

def getReposFromFile (file):
    repos = []
    for line in open(file, "r").read().strip().split("\n"):
        if (not line) or line.startswith("#"):
            continue

        repos.append(getRepoFromLine(line))
    return repos

def getRepoFromLine (line):
    parts = line.split("#")[0].split(" ")

    repo_type = parts.pop(0)
    url       = parts.pop(0)
    options   = None
    if url[0] == "[" and url[-1] == "]":
        options = url
        url     = parts.pop(0)

    dist      = parts.pop(0)
    component = None
    if parts:
        component = parts.pop(0)

    return Repo(
        type      = repo_type,
        options   = options,
        url       = url,
        dist      = dist,
        component = component
    )

class Package (object):
    def __init__ (self, data):
        self.package = data["Package"]
        self.description = data.get("Description", "")
        self.maintainer = data.get("Maintainer", "")
        self.section = data.get("Section", "")
        self.md5sum = data.get("MD5sum", "")
        self.installedSize = data.get("Installed-Size", 0)
        self.depends = [depend.strip() for depend in data.get("Depends", "").split(",")]
        self.version = data.get("Version", "")
        self.architecture = data.get("Architecture", "")
        self.size = data.get("Size", 0)
        self.status = data.get("Status", "")
        self.priority = data.get("Priority", "")

    def __repr__ (self):
        return 'Package({"Package": %r, "Description": %r, "Maintainer": %r, "Section": %r, "MD5sum": %r, "Installed-Size": %s, "Version": %r, "Architecture": %r, "Size": %r, "Status": %r, "Priority": %r})' % (self.package, self.description, self.maintainer, self.section, self.md5sum, self.installedSize, self.version, self.architecture, self.size, self.status, self.priority)

def parsePackageList (packageData):
    packages = []
    currentPackageData = {}
    key = None
    value = None
    for line in packageData.strip().split("\n"):
        if line.startswith(" "):
            value = value + " " + line
        else:
            if key is not None:
                currentPackageData[key] = value
            if ":" in line:
                parts = line.split(":")
                key = parts[0]
                if len(parts) > 1:
                    value = parts[1].strip()
                else:
                    value = ""

        if len(line.strip()) == 0:
            if "Package" in currentPackageData:
                packages.append(Package(currentPackageData))
            currentPackageData = {}

    # 8 December 2017: append any leftover package data
    if currentPackageData and "Package" in currentPackageData:
        packages.append(Package(currentPackageData))

    return packages

def getSystemPackages (context):
    with open(context.get_path(DPKG_STATUS)) as dpkgStatus:
        return parsePackageList(dpkgStatus.read())

def urlopen (url):
    ((username, password), url) = extract_auth(url)
    request = urllib.request.Request(url)

    if not (username is None and password is None):
        base64string = base64.b64encode("{}:{}".format(username, password).encode()).decode()
        request.add_header("Authorization", "Basic %s" % base64string)

    return urllib.request.urlopen(request)

def extract_auth (url):
    parsed = urlparse(url)
    url = "{}://{}{}{}".format(parsed.scheme, parsed.hostname, ":{}".format(parsed.port) if parsed.port else "", parsed.path)
    return ((parsed.username, parsed.password), url)

class Changes (object):
    def __init__ (self, changes):
        self.changes = changes or []

    def __repr__ (self):
        return "Changes({})".format(self.changes)

    def get_changes_since (self, version):
        changes_since = []
        version = Version(version)
        for changes in self.changes:
            if Version(changes.version) <= version:
                break
            changes_since.append(changes)
        return changes_since

class Changeset (object):
    def __init__ (self, package, version, changes=None):
        self.package = package
        self.version = version
        self.changes = changes or []

    def __repr__ (self):
        return "Changeset({}, {})".format(self.package, self.version)

def parseChangelog (changelog):
    changes = []
    changeset = None

    for line in changelog.split("\n"):
        line = line.strip()
        if not line:
            continue

        if changeset is None:
            (package, version, etc) = line.split(" ", 2)
            version = version[1:-1]
            changeset = Changeset(package, version)
        elif line.startswith("*"):
            changeset.changes.append(line[1:].strip())
        elif line.startswith("--"):
            changes.append(changeset)
            changeset = None
        elif changeset.changes:
            changeset.changes[-1] = changeset.changes[-1] + " " + line

    return Changes(changes)

@total_ordering
class Version (object):
    def __init__ (self, version_string):
        self.parts = [VersionPart(part) for part in version_string.split(".")]

    def __lt__ (self, other):
        for i in range(0, max(len(self.parts), len(other.parts))):
            my_part = self.parts[i] if len(self.parts) > i else VersionPart.ZERO
            their_part = other.parts[i] if len(other.parts) > i else VersionPart.ZERO
            if my_part == their_part:
                continue

            return my_part < their_part
        return False

    def __eq__ (self, other):
        for i in range(0, max(len(self.parts), len(other.parts))):
            my_part = self.parts[i] if len(self.parts) > i else VersionPart.ZERO
            their_part = other.parts[i] if len(other.parts) > i else VersionPart.ZERO
            if not my_part == their_part:
                return False
        return True

@total_ordering
class VersionPart (object):
    def __init__ (self, value):
        try:
            self.value = int(value)
            self.is_number = True
        except ValueError:
            self.value = value
            self.is_number = False

    def __lt__ (self, other):
        if self.is_number and other.is_number:
            return self.value < other.value
        return str(self.value) < str(other.value)

    def __eq__ (self, other):
        if self.is_number and other.is_number:
            return self.value == other.value
        return str(self.value) == str(other.value)

VersionPart.ZERO = VersionPart(0)
