import threading

# Popen.wait()/poll() isn't thread safe: http://bugs.python.org/issue21291
# We need -three- threads: a stdout reader, stderr reader, and poll thread.
# That way the returncode attribute is thread-safely handled, as long
# as Pipes.wait() is used instead of Popen.wait()
class Pipes (object):
    def __init__ (self, process, out_callbacks=None, err_callbacks=None):
        self.process = process
        self.exited = threading.Event()
        self.poller = self.__start_poller()
        
        self.err_callbacks = err_callbacks if err_callbacks else []
        if process.stderr:
            self.err_reader = self.__start_reader(process.stderr, self.err_callbacks)
        
        self.out_callbacks = out_callbacks if out_callbacks else []
        if process.stdout:
            self.out_reader = self.__start_reader(process.stdout, self.out_callbacks)
    
    def wait (self):
        self.poller.join()

        if self.err_reader:
            self.err_reader.join()

        if self.out_reader:
            self.out_reader.join()
        
        return self.process.returncode
    
    def __start_poller (self):
        def poller ():
            self.process.wait()
            self.exited.set()
        
        poller = threading.Thread(target=poller)
        poller.daemon = True
        poller.start()
        return poller
    
    def __start_reader (self, source, callbacks):
        def pipe ():
            newlines = 0
            while True:
                line = source.readline().decode().rstrip("\r\n")
                if line == "":
                    if self.exited.isSet():
                        break
                    newlines = newlines + 1
                else:
                    for callback in callbacks:
                        for i in range(0, newlines):
                            callback("")
                        newlines = 0
                        callback(line)
        
        reader = threading.Thread(target=pipe)
        reader.daemon = True
        reader.start()
        return reader
