import os

from urllib.request import urlopen
from urllib.error import HTTPError
from urllib.parse import urlparse
from datetime import datetime
from subprocess import check_output
import calendar

from .config import options
from .cranium import Cranium
from .packages import Repo, getReposFromFile, extract_auth

LIMELM_DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S"

DEFAULT_REPO_URL = "https://repositories.catturavideo.com/capturecast/"

REPO_KEY = """-----BEGIN PGP PUBLIC KEY BLOCK-----

mQINBFpyLZYBEACnjaEzspoxKQi4OroKCKEW5yY60SdL8Qmd2yG+fVoVPGprLurO
5g+349p9bwb9vzqZls5s7OFWbWYlk0/r6xZERYXCFEkZLA/5z5yeXErUUAcLGqks
IXzuY+7x8y9SovhN16Zfx0swlkoF0GqviCEUeXA9tbrCL/705AzJONqCm7aC+hJw
2w112XnJgZZE9x8SZaL3YjhH/d06MUbut7zNJYb1Lj01T6xe0QAwv6M6V67ibVS/
9VunTL3aOMteBebbkHoyJJ4+Y72M9ZU3QhOUFxS0sJ0xhduMuMxAPMj+vcX0Y2ik
W4JlYe4Q9FEW2bT03gxM1w5PGNf3xphqb/eSEg3RF/isC1zUIFq/m/BeUEzFdkSv
xmkaYNbY9uC9YxjDEG4I05tHBa2PQMvbCEzIuLK/S4itgpqQ/agfbcAGg7AuRyzr
Id7tJOV1WCxOk3Wjnfr+LHSSL489sUZa2KVG3ScO5IF+Q9EIh2tG46yha3EhDfuS
fYJuFwB6X1CPkmD9zbL+j2nr8BUVVPSTYpg2dLPyzUTJQj7GqBmyG20TrXmcOF0T
K3q+6HT/UTS8TjuZvOZ2gGKFxEOoDZCk7KyS1XX1MzsjUcbKSvUFg+AbMIHd2V9Y
1HdY07dEMj/iC0p6PeIV0mzoEhirHXbP34rVW+DYckvV4Vjouh15DnZb5QARAQAB
tEVHcmVnb3J5IE1hcmNvIChDYXB0dXJlQ2FzdCBQcm8gU2lnbmluZyBLZXkpIDxn
bWFyY29AY2F0dHVyYXZpZGVvLmNvbT6JAk4EEwEKADgWIQR2kPo41yBcNu++oMiN
FDOuAut4MwUCWnItlgIbAwULCQgHAgYVCgkICwIEFgIDAQIeAQIXgAAKCRCNFDOu
Aut4M6BuEACNA+y86393oJeFpOhFrdEvukYhiaj+NdYS3kWzrdIRytivPHeak2UC
mu/Ojf0Nts3loExYFHAWtch/jmsVGlHbmS5Rll3MjCcFRaMp9kQE7wGgvWdvdyX+
UAcshmuGZPfz1ADqfdoYVg8sWGpLtV+PwnFDflTGxyKbxzVBeSPxaWv3I96lAZHy
JRdcZ/tNSms9fxd/5ewklBu+xO7FEefe/j+vP0hAxvdFIWWyyDIq6H9D/PwczVA/
UQFt4S2RkUsdbvsVNInnZkLOfB1zOdsdAUpSOlMQf0QGXFveVg/vafo1KdYg3hf2
n8nA+ybxNPUp1EPZuO8QE7RoDiF2+b8By3GM+fSY4oLHFh32d3qxVGwsDFJb0Sb4
tQNK9FNlMLxuajApLh+mAPtQHX9o328K1+pX7wOPl+gC1Dz8yLfVa6Y+xbZL1rCY
32b1JN7en+GidCXIEsNi19IJ4H2udvziE72zkcGYRkJq2Fj66XFWAH7fdRxcEgKV
ayyY9SLAvxqz5hdtWbwzhlWWxhPR7nGc8GMhZs2xtXL9IkSQ9Xcd/bwTBK+mMU2+
117h2/whXo8QC3ZvuNujbfbv7n70V5ACv8daGMa9/97DcwKg+1KIVVxi0c+iGJdA
rDiZlTOMUVupR9m1x5eGP/qAS/oy86d98QSN9rEewsyyI6gLuQvl1Q==
=pM+6
-----END PGP PUBLIC KEY BLOCK-----
"""

REPOS = {
    "capturecast": {
        "file": "/etc/apt/sources.list.d/cattura.list",
        "path": "/"
    },
    "extra": {
        "file": "/etc/apt/sources.list.d/cattura-extra.list",
        "url": "https://repositories.catturavideo.com/extra/",
        "path": "/"
    },
    "extra-auto": {
        "file": "/etc/apt/sources.list.d/cattura-extra-auto.list",
        "url": "https://repositories.catturavideo.com/extra/auto/",
        "path": "/"
    },
    "extra-native": {
        "file": "/etc/apt/sources.list.d/cattura-extra-native.list",
        "url": "https://repositories.catturavideo.com/extra-native/",
        "path": "",
        "component": "main"
    }
}

def get_repo_configuration (context):
    """Extract the repo configuration of the main CaptureCast repo.
    This will be a dictionary with "url" and "authentication" keys.
    "authentication" is a tuple of (unit_id, token).
    "authentication" will not exist if the unit has no registration details.
    """
    repo_file = context.get_path(REPOS["capturecast"]["file"])
    if not os.path.isfile(repo_file):
        context.logger.warn("apt list file {} does not exist, cannot determine repo configuration".format(repo_file))
        return
    
    for repo in getReposFromFile(repo_file):
        details = extract_auth(repo.url)
        if details[0][0]:
            return {
                "authentication": details[0],
                "url": details[1],
                "repo": repo
            }
        return {
            "url": details[1],
            "repo": repo
        }

def repo_requires_authentication (repo_configuration):
    repo = Repo("deb", None, repo_configuration["url"], "/")

    try:
        repo.packages()
    except HTTPError as e:
        if e.code == 403:
            return True
    
    return False

def repo_url_with_authentication (context, repo_configuration):
    """Given repo configuration (returned from get_repo_configuration),
    return a URL with the repo authentication added in.
    This is suitable for being inserted into a sources.list file.
    """
    parsed = urlparse(get_true_url_for_repo_configuration(repo_configuration))

    authentication = ""
    if "authentication" in repo_configuration:
        authentication = "{}:{}@".format(
            repo_configuration["authentication"][0],
            repo_configuration["authentication"][1]
        )

    return "{}://{}{}{}".format(
        parsed.scheme,
        authentication,
        parsed.netloc,
        parsed.path
    )

def get_update_channel_from_repo_configuration (repo_configuration):
    return urlparse(repo_configuration["url"]).path[1:]

def get_specified_repos (context, repos=[]):
    """Given a list of repository names (keys in REPOS), return repo information
    for each of them.
    This is mainly used for check_updates and update_platform tasks.
    """
    cattura_repos = {}
    for key, value in REPOS.items():
        if repos and not key in repos:
            continue

        full_path = context.get_path(value["file"])
        if not os.path.isfile(full_path):
            context.logger.warn("apt list file {} does not exist, ignoring".format(value["file"]))
            continue

        if not value.get("update_check_enabled", True):
            context.logger.info("update checks are disabled for repo {}, ignoring".format(key))
            continue

        cattura_repos[key] = getReposFromFile(full_path)
    return cattura_repos

def get_full_repo_url (repo_name):
    if repo_name in options["repos"]:
        repo_name = options["repos"][repo_name]

    if not (repo_name.startswith("http://") or repo_name.startswith("https://")):
        repo_name = "{}{}".format(DEFAULT_REPO_URL, repo_name)

    return repo_name

def discover_cranium (repo_url):
    """Attempts to discover a Lemonade Stand (Super Cranium) instance at the
    given URL.
    """
    if not repo_url:
        return None

    while True:
        cranium = Cranium(repo_url)

        try:
            cranium.info()
            return cranium
        except Exception:
            if urlparse(repo_url).path == "/":
                return None
            repo_url = repo_url[:repo_url.rindex("/")]

def get_registration_from_cranium (cranium, repo_configuration, logger):
    user_details = cranium.get_user("me")

    try:
        user_details.update(cranium.get_unit("{}/activation".format(repo_configuration["authentication"][0])))
    except Exception:
        logger.warn("Failed to get activation details")
        user_details["valid"] = False

    return Registration(user_details, repo_configuration)

def get_true_url_for_repo_configuration (repo_configuration):
    """Given a repo configuration, determine the true URL
       after following redirect, e.g. http->https redirect.
    """
    return urlopen(repo_configuration["url"]).geturl()

def get_distribution_name ():
    """Return distribution name (e.g. xenial, bionic)
    """
    return check_output(["lsb_release", "-sc"]).decode().strip()

class Registration (object):
    def __init__ (self, data, repo_configuration):
        self.licensed = True
        self.key = data["pkey"]["key"]
        self.email = data["pkey"]["email"]
        self.created = _parse_datetime(data["pkey"]["created"])
        self.activated = None
        self.serial = None
        self.valid = data["valid"]
        self.update_channel = None

        if data.get("activation", None):
            self.activated = _parse_datetime(data["activation"]["date"])
            self.serial = data["activation"]["extra_data"]

        if repo_configuration and "url" in repo_configuration:
            self.update_channel = get_update_channel_from_repo_configuration(repo_configuration)

        features = data["pkey"]["features"]["feature"]
        self.fields = {feature["name"]: feature["value"] for feature in features}
        self.licensed_to = _find_feature_value(features, "Licensed to")
        self.expiration_date = _parse_datetime(_find_feature_value(features, "Expiration date"))

def _find_feature_value (features, name):
    try:
        return [feature for feature in features if feature["name"] == name][0]["value"]
    except IndexError:
        return None

def _parse_datetime (value):
    if value is None:
        return None

    return calendar.timegm(datetime.strptime(value, LIMELM_DATETIME_FORMAT).timetuple()) * 1000
