import logging
from datetime import datetime

from ..context import Context
from ..registration import discover_cranium, get_repo_configuration, get_registration_from_cranium 

logger = logging.getLogger(__name__)

INVALID_REGISTRATION = {"licensed": False}

def __call__ (config):
    context = Context(config, logger)
    repo_configuration = get_repo_configuration(context)
    if not repo_configuration:
        logger.info("No Cattura repository details found for this recorder")
        return INVALID_REGISTRATION

    if not "authentication" in repo_configuration:
        logger.info("Cattura repository details exist for this recorder, but do not contain authentication details")
        return INVALID_REGISTRATION
    
    cranium_url = config.registration_url if config.registration_url else repo_configuration["url"]
    cranium = discover_cranium(cranium_url)
    if not cranium:
        logger.warn("Could not detect lemonade stand instance at %s", cranium_url)
        return INVALID_REGISTRATION
    
    cranium.token = repo_configuration["authentication"][1]
    registration = get_registration_from_cranium(cranium, repo_configuration, logger)
    log_registration(registration)
    return registration

def log_registration (registration):
    logger.info("Registration Details:")
    logger.info(" * Licensed to: %s (%s)", registration.licensed_to, registration.email)
    logger.info(" * License key: %s", registration.key)
    logger.info(" * Serial: %s", registration.serial)
    logger.info(" * License valid: %s", registration.valid)
    logger.info(" * Licensed since: %s", format_timestamp(registration.created))
    logger.info(" * Activated on: %s", format_timestamp(registration.activated))
    logger.info(" * Expires on: %s", format_timestamp(registration.expiration_date))
    logger.info(" * Update channel: %s", registration.update_channel)

def format_timestamp (timestamp):
    if timestamp is None:
        return "Never"

    return datetime.fromtimestamp(timestamp / 1000)
