import os
import logging

from ..config import options, defaults
from ..context import Context
from ..helpers import resolve_add_remove_features

logger = logging.getLogger(__name__)

def __call__ (config):
    context = Context(config, logger)
    # Install linux headers
    for kernel_version in scan_linux_kernels(context):
        try:
            logger.info("Installing Kernel Headers for kernel version: {}".format(kernel_version))
            context.install("linux-headers-{}".format(kernel_version))
        except Exception as e:
            logger.error("Unable to install headers for kernel {}: {}".format(kernel_version, e))
    
    # install user-selected drivers
    (add_drivers, remove_drivers) = resolve_add_remove_features(
        available_features = options["drivers"],
        selected_features = context.config.drivers,
        default_features = get_standard_drivers()
    )
    
    for driver in add_drivers:
        try:
            logger.info("Installing Driver: {}".format(driver["title"]))
            context.install(driver["packages"])
        except Exception as e:
            logger.warn("Failed to install driver {}: {}. Will roll back.".format(driver["title"], e))
            context.remove(driver["packages"])

    if len(remove_drivers) > 0:
        logger.info("Removing Drivers: {}".format(", ".join([driver["title"] for driver in remove_drivers])))
        context.remove(*[driver["packages"] for driver in remove_drivers])

def get_standard_drivers ():
    return defaults["drivers"]

def scan_linux_kernels (context):
    VMLINUZ = "vmlinuz-"
    return [entry[len(VMLINUZ):] for entry in os.listdir(context.get_path("/boot")) if entry.startswith(VMLINUZ)]