import os
import logging
from bundle import open_entry

from ..config import defaults
from ..context import Context
from ..commands import chroot, bash

from ..registration import repo_requires_authentication, get_full_repo_url, REPO_KEY, REPOS, discover_cranium, get_repo_configuration, repo_url_with_authentication, get_registration_from_cranium, get_update_channel_from_repo_configuration

from turbo_activate import TurboActivate, PDetsFromByteArray, USER, OK, ActivateOptions

logger = logging.getLogger(__name__)

CAPTURECAST_PRO_GUID = "50d602e6586edb97afc8e4.97968303" # for TurboActivate integration

def __call__ (config):
    context = Context(config, logger)

    # Repository configuration.
    # Default to the default repository, of course.
    # - authentication is a tuple of (unit_id, token)
    repo_configuration = {
        "url": get_full_repo_url(defaults["repo"]),
        "repo": None
    }
    
    cranium = discover_cranium(config.registration_url) if config.registration_url else None

    # Registration details that will be returned.
    registration = None

    # The file our repo details are stored in.
    cattura_apt_file = context.get_path(REPOS["capturecast"]["file"])

    # Read existing authentication details from repo.
    if os.path.exists(cattura_apt_file):
        repo_configuration = get_repo_configuration(context)
        if repo_configuration:
            if not config.registration_url:
                cranium = discover_cranium(repo_configuration["url"])

            if "authentication" in repo_configuration:
                cranium.token = repo_configuration["authentication"][1]
                registration = get_registration_from_cranium(cranium, repo_configuration, logger)

                if not registration.valid:
                    logger.warn("Existing registration appears to be invalid, ignoring")
                    registration = None

    # Get the full repo url of the configured repository.
    # If specified on the config, this will always override whatever is already
    # configured on the system. Also, if cranium is detected from the repo configuration,
    # that is overridden too (but not if the registration url is specified separately).
    if context.config.repo:
        repo_configuration["url"] = get_full_repo_url(context.config.repo)
        if not config.registration_url:
            cranium = discover_cranium(repo_configuration["url"])

    if not cranium:
        cranium = discover_cranium(repo_configuration["url"])

    if not repo_requires_authentication(repo_configuration):
        logger.info("Skipping registration because it is not required for this repository")
        cranium = None

    # New unit registration - register through cranium
    if cranium:
        if not registration:
            unit_info = register_recorder(context, cranium)
            registration = unit_info["registration"]
            repo_configuration["authentication"] = (unit_info["unit"]["id"], unit_info["token"])
        # Existing registration - just re-use it
        else:
            logger.info("Existing registration information found for this recorder, so re-using it")

    logger.info("Adding Cattura signing key")
    context.call_privileged(chroot(config.target, bash("echo '{}' | apt-key add -".format(REPO_KEY))))

    logger.info("Adding Cattura repository to apt")
    apt_source = ["deb"]

    if repo_configuration["repo"] and repo_configuration["repo"].options:
        apt_source.append(repo_configuration["repo"].options)

    apt_source.append(repo_url_with_authentication(context, repo_configuration))
    apt_source.append("/")

    context.write_privileged(" ".join(apt_source), cattura_apt_file)

    # update apt-get
    context.apt_update()

    # Return registration details.
    if registration:
        registration.update_channel = get_update_channel_from_repo_configuration(repo_configuration)
        return registration

def register_recorder (context, cranium):
    if not cranium:
        logger.info("Skipping registration since there doesn't seem to be a valid registration endpoint...")
        return None

    unit_id = context.get_hostname()
    logger.info('Registering unit with id "{}"'.format(unit_id))

    if context.config.dry:
        return {
            "unit": {"id": unit_id},
            "token": "jeff",
            "registration": None
        }

    cranium.token = context.config.key
    cranium_user = None

    # If we are interactively asking for the token, then loop until
    # we are given a valid one. Otherwise, we just throw an exception
    # if it's invalid.
    if not cranium.token:
        while True:
            try:
                cranium.token = context.input(" >> Enter registration key: ")
                cranium_user = cranium.get_user("me")
                break
            except Exception:
                logger.warn("Invalid registration key provided")
    else:
        try:
            cranium_user = cranium.get_user("me")
        except Exception:
            raise Exception("Invalid registration key provided")

    prospective_unit_id = unit_id
    unit = None
    try:
        unit = cranium.add_unit({
            "url": "http://{}".format(cranium.client_info()["ip"])
        }, unit_id=prospective_unit_id)
    except Exception as e:
        logger.info('Failed to create unit "{}" ({}); assuming unit already exists'.format(prospective_unit_id, e))
        unit = cranium.get_unit(prospective_unit_id)

    # If we're a LimeLM key, use the TurboActivate flow
    if "pkey" in cranium_user:
        return register_with_turbo_activate(cranium, unit)
    else:
        token = cranium.add_unit_token(unit["id"])
        return {
            "unit": unit,
            "token": token["inboundAccessToken"],
            "registration": get_registration_from_cranium(cranium, {
                "authentication": (unit["id"], token["inboundAccessToken"])
            }, logger)
        }

def register_with_turbo_activate (cranium, unit):
    turbo_activate = initialize_turbo_activate()

    # Generate activation request for TurboActivate, including our unit id.
    turbo_activate.CheckAndSavePKey(cranium.token, USER)
    activation_request = turbo_activate.ActivationRequestToFile("TurboActivateRequest.xml", ActivateOptions(unit["id"]))
    activation_response = None

    with open("TurboActivateRequest.xml") as activation_request:
        activation_response = cranium.activate_unit(unit["id"], activation_request.read())

    # TurboActivate units use the LimeLM product key to authenticate
    return {
        "unit": unit,
        "token": activation_response["token"],
        "registration": get_registration_from_cranium(cranium, {
            "authentication": (unit["id"], cranium.token)
        }, logger)
    }

def initialize_turbo_activate ():
    with open_entry("TurboActivate.dat", "rb") as pdets_file:
        pdets_data = pdets_file.read()
        if PDetsFromByteArray(pdets_data, len(pdets_data)) != OK:
            raise Exception("PDetsFromByteArray failed")
    return TurboActivate(CAPTURECAST_PRO_GUID)
