from bundle import open_entry_as_file
from ctypes import CDLL, Structure, c_uint, c_char_p, sizeof, byref

from . import errors

with open_entry_as_file("turbo_activate/libTurboActivate.so") as so_file:
    __libTurboActivate = CDLL(so_file.name)

__EXPORTS = ["GetHandle", "Activate", "ActivationRequestToFile", 
             "ActivateFromFile", "CheckAndSavePKey", "Deactivate",
             "DeactivationRequestToFile", "GetExtraData", "GetFeatureValue",
             "GetPKey", "IsActivated", "IsGenuine", "IsGenuineEx",
             "GenuineDays", "IsProductKeyValid", "SetCustomProxy",
             "TrialDaysRemaining", "UseTrial", "UseTrialVerifiedRequest",
             "UseTrialVerifiedFromFile", "ExtendTrial", "PDetsFromPath",
             "PDetsFromByteArray", "SetCustomActDataPath", "IsDateValid"]

GetHandle = None # shut up pyflakes

# success/error codes
OK = 0
FAIL = 1

# flags for IsGeniuneEx()
SKIP_OFFLINE = 1
OFFLINE_SHOW_INET_ERR = 2

# flags for UseTrial() and CheckAndSavePKey()
SYSTEM = 1
USER = 2

# flags for UseTrial()
DISALLOW_VM = 4
DISALLOW_SANDBOX = 8
UNVERIFIED_TRIAL = 16
VERIFIED_TRIAL = 32

# structs
class GENUINE_OPTIONS (Structure):
    _fields_ = [
        ("nLength", c_uint),
        ("flags", c_uint),
        ("nDaysBetweenChecks", c_uint),
        ("nGraceDaysOnInetErr", c_uint),
    ]

class ACTIVATE_OPTIONS (Structure):
    _fields_ = [
        ("nLength", c_uint),
        ("sExtraData", c_char_p),
    ]

def ActivateOptions (extra_data):
    return ACTIVATE_OPTIONS(sizeof(ACTIVATE_OPTIONS()), c_char_p(extra_data.encode()))

class TurboActivateException (Exception):
    def __init__ (self, message, code):
        Exception.__init__(self, "[Error {}] {}".format(code, message))
        self.code = code

class TurboActivate (object):
    def __init__ (self, versionGUID):
        self._handle = GetHandle(versionGUID.encode())
        if self._handle == 0:
            raise TurboActivateException("Failed to get the handle for specified version GUID", 0)
    
    @classmethod
    def _add_function (cls, name, func):
        def __call_func (self, *args, **kwargs):
            retval = func(self._handle, *[_translate_arg(arg) for arg in args], **kwargs)
            if retval > 0:
                raise TurboActivateException(strerror(retval), retval)
            
            return retval
        setattr(cls, name, __call_func)

# Export error codes
__error_messages = {}
for key, value in vars(errors).items():
    if not key.startswith("E_"):
        continue
    
    globals()[key] = value[0]
    __error_messages[value[0]] = value[1]

def strerror (code):
    if code == 1:
        return "An error occurred."
    
    if code in __error_messages:
        return __error_messages[code]
    
    raise Exception("Unknown error code: {}".format(code))

for name in __EXPORTS:
    func = getattr(__libTurboActivate, "TA_{}".format(name))
    globals()[name] = func
    TurboActivate._add_function(name, func)

def _translate_arg (arg):
    if isinstance(arg, Structure):
        return byref(arg)
    elif isinstance(arg, str):
        return arg.encode()
    return arg